package de.fraunhofer.iem.dataprovider.repository.service

import de.fraunhofer.iem.dataprovider.configuration.OpenCodeApiProperties
import de.fraunhofer.iem.dataprovider.gitlab.service.OpenCodeGitlabApi
import de.fraunhofer.iem.dataprovider.logger.getLogger
import de.fraunhofer.iem.dataprovider.repository.dto.RepositoryConsentDto
import de.fraunhofer.iem.dataprovider.repository.dto.RepositoryCreateDto
import de.fraunhofer.iem.dataprovider.repository.dto.ValidateUserDto
import de.fraunhofer.iem.dataprovider.repository.entity.RepositoryEntity
import de.fraunhofer.iem.dataprovider.repository.repository.RepositoryRepository
import io.ktor.client.*
import io.ktor.client.call.*
import io.ktor.client.engine.cio.*
import io.ktor.client.plugins.contentnegotiation.*
import io.ktor.client.plugins.cookies.*
import io.ktor.client.request.*
import io.ktor.http.*
import io.ktor.serialization.kotlinx.json.*
import kotlinx.serialization.json.Json
import org.springframework.stereotype.Service
import java.util.*

@Service
class RepositoryService(
    private val repositoryRepository: RepositoryRepository,
    private val openCodeGitlabApi: OpenCodeGitlabApi,
    private val openCodeApiProperties: OpenCodeApiProperties
) {
    private val logger = getLogger(javaClass)
    private val httpClient = HttpClient(CIO) {
        install(HttpCookies)
        install(ContentNegotiation) {
            json(
                Json { ignoreUnknownKeys = true }
            )
        }
    }

    /**
     * Either creates or returns a repository entity based upon its
     * opencode repository id.
     */
    fun getOrCreate(gitRepository: RepositoryCreateDto): RepositoryEntity {
        logger.info("Checking if repository with ${gitRepository.id} exists")
        var repo = repositoryRepository.findByProjectId(gitRepository.id)
        if (repo == null) {
            logger.info("Creating repository with id ${gitRepository.id}")
            repo = repositoryRepository.save(gitRepository.toDbObject())
        }
        return repo
    }

    /**
     * Queries the gitlab api to get the repo name, url, and id.
     */
    fun getRepositoryInfo(projectId: Long): RepositoryCreateDto {
        return openCodeGitlabApi.getRepositoryInfo(projectId)
    }

    fun findRepoByID(id: UUID): RepositoryEntity? {
        val repository = repositoryRepository.findById(id)
        if (repository.isEmpty) {
            return null
        }
        return repository.get()
    }

    fun findRepoById(projectId: Long): RepositoryEntity? {
        return repositoryRepository.findByProjectId(projectId)
    }

    fun getAllRepositories(): List<RepositoryEntity> {
        return repositoryRepository.findAll()
    }

    fun updateVisualizationConsent(repositoryConsentDto: RepositoryConsentDto) {
        val repo = findRepoById(repositoryConsentDto.projectId)
        if (repo != null) {
            repo.visualizationConsent = repositoryConsentDto.consent
            logger.info("Updated visualization consent $repositoryConsentDto")
            repositoryRepository.save(repo)
        } else {
            logger.error(
                "Could not update visualization consent because repo" +
                    "was not found $repositoryConsentDto"
            )
        }
    }

    fun getAllRepositoriesWithConsent(): List<RepositoryEntity> {
        return repositoryRepository.findByVisualizationConsentTrue()
    }

    suspend fun validateUser(validateUserDto: ValidateUserDto) {
        val userCookie = httpClient.get(openCodeApiProperties.auth) {
            url {
                appendPathSegments(validateUserDto.b)
                parameters.append("apiKey", openCodeApiProperties.authApiKey)
            }
        }
        println(userCookie)
        println(userCookie.headers)
        println(userCookie.body<Any>())
        val setCookies = userCookie.setCookie()
        println(setCookies)

        val me = httpClient.get(openCodeApiProperties.auth) {
            url {
                appendPathSegments("me")
                parameters.append("apiKey", openCodeApiProperties.authApiKey)
            }
        }

        println(me)
        println(me.body<Any>())
    }

}
